#include "haar.h"

#ifndef haar_c
#define haar_c

Haar::~Haar () {
}

int raise_to_power (int value, int power) {
  int i, out_val;
  out_val = 1;
  if (power > 0) {
    for (i=1; i<=power; i++) {
      out_val *= value;
    }
  }
  return out_val;
}

void Haar::create () {
  int k,x;
  float scale_x;
  int m,n;
  int max_n, min_n;
  float bound1, bound2, bound3;
  data[0][0] = 1;
  if (size > 1) {
    for (m=0; m<p; m++) {
      if (m == 0) {
	max_n = 1;
	min_n = 0;
      } else {
	max_n = raise_to_power(2,m);
	min_n = 1;
      }
      for (n=min_n; n<=max_n; n++) {
	// find k
	k = raise_to_power(2,m) + (n-1);
	// set bounds:
	bound1 = ((float) n-1)/(raise_to_power(2, m));
	bound2 = (((float) n) - .5)/(raise_to_power(2, m));
	bound3 = ((float) n)/(raise_to_power(2, m));
	for (x=0; x<size; x++) {
	  // scaled x
	  scale_x = ((float) x)/size;
	  // create array:
	  if (k == 0) {
	    data[k][x] = 1;
	  } else if (bound1 <= scale_x && scale_x < bound2) {
	    data[k][x] = sqrt(raise_to_power(2, m));
	  } else if (bound2 <= scale_x && scale_x < bound3) {
	    data[k][x] = -1*sqrt(raise_to_power(2, m));
	  } else {
	    data[k][x] = 0;
	  }
	}
      }
    }
  }
}

// scale waveform to haar transform input
// ie: positive values to 1, others (incl. 0) to -1
// note: array must be same size as transform
void Haar::scale_to (float * array) {
  int i;
  for (i=0; i<size; i++) {
    array[i] = (array[i] > 0) ? 1 : -1;
  }
}

// scale haar inverse-transform output to waveform
// ie: positive values to 1, others (incl. 0) to 0
// note: array must be same size as transform
void Haar::scale_from (float * array) {
  int i;
  for (i=0; i<size; i++) {
    array[i] = (array[i] > 0) ? 1 : 0;
  }
}

// threshold array (transformed data)
// ie: takes in array (same size as transform)
// returns it "cleaned up"
void Haar::threshold (float * array) {
  float tmp_array[size];
  int i, count;
  int div_by = 1;          // value to divide by
  float mult_by = 1; // value to multiply by
  int sec_num = 1;         // section number
  float section_total = 0;
  float new_thresh = 0;

  // copy
  for (i=0; i<size; i++) {
    tmp_array[i] = array[i];
  }

  for (i=0; i<size; i++) {
    if (i == sec_num) {
      // reached a new section:
      // check to see if more than 1/2 have been filled in
      if (section_total > ((float) sec_num*size)/2) {
	// backtrack through this section:
	if (sec_num > 1) {
	  for (count=i-1; count>i-sec_num; count--) {
	    //	    array[count] = (tmp_array[i]>0) ? 
	  }
	}
      }
      // get ready for next section
      div_by *= 2;
      mult_by *= sqrt(2);
      sec_num = sec_num * 2;
      section_total = 0;
    }

    new_thresh = (((float) threshold_value / (float) div_by) * mult_by);
    //    cout << "compare " << Abs(array[i]) << " to " << new_thresh << " (" << threshold_value << ")\n";
    
    if (Abs(array[i]) >= new_thresh) {
    //    if (Abs(array[i]) >= threshold_value) {
      array[i] = (Abs(array[i])/array[i]) * size;
    } else {
      array[i] = 0;
    }
  }
}

#endif
