#include "hadamard.h"

#ifndef hadamard_c
#define hadamard_c

Hadamard::~Hadamard () {
}

void Hadamard::create () {
  int n,i,j;
  data[0][0] = 1;
  if (size > 1) {
    for (n=2; n<=size; n=n*2) {
      for (i=0; i<n; i++) {
	for (j=0; j<n; j++) {
	  if (j >= n/2) {
	    data[i][j] = data[i][j-(n/2)];
	  } else if (i >= n/2) {
	    data[i][j] = data[i-(n/2)][j];
	  }
	  if (j >= n/2 && i >= n/2) {
	    data[i][j] = -1 * data[i][j];
	  }
	}
      }
    }	  
  }
}

// scale waveform to hadamard transform input
// ie: positive values to 1, others (incl. 0) to -1
// note: array must be same size as Hadamard transform
void Hadamard::scale_to (float * array) {
  int i;
  for (i=0; i<size; i++) {
    array[i] = (array[i] > 0) ? 1 : -1;
  }
}

// scale hadamard inverse-transform output to waveform
// ie: positive values to 1, negative to 0
// if zero, then alternate between 1 and 0
// note: array must be same size as transform
void Hadamard::scale_from (float * array) {
  int i;
  int to_zero = 0;
  for (i=0; i<size; i++) {
    if (array[i] > 0) {
      array[i] = 1;
    } else if (array[i] < 0) {
      array[i] = 0;
    } else {
      // array[i] is zero, so either assing to 1 or 0
      array[i] = (to_zero++ % 2) ? 1 : 0;
    }
    // OLD WAY:
    // - if positive, assign to 1,
    // - else (incl. 0), assign to 0
    //    array[i] = (array[i] > 0) ? 1 : 0;
  }
}

// threshold array (transformed data)
// ie: takes in array (same size as transform)
// returns it "cleaned up"
void Hadamard::threshold (float * array) {
  int i;
  for (i=0; i<size; i++) {
    if (Abs(array[i]) >= threshold_value) {
      // old way: assigned value to +/- size of array
      //      array[i] = (Abs(array[i])/array[i]) * size;
      // new way: assign values to +/- 1
      array[i] = (Abs(array[i])/array[i]);
    } else {
      array[i] = 0;
    }
  }
}

#endif
